#include "haplotypetest.h"

#define	CHI_EPSILON     0.000001
#define	CHI_MAX     99999.0

/* Gamma function */
extern double gammq(double, double);

/* Define functions */
double         calc_p_value_by_chi_square(int, double);
double         calc_statistic_by_chi_square(int, double);

double         calc_p_value_by_chi_square(int df, double x)
{
  double   p_value, a, y;

  a = (double)df / 2.0;
  y =         x  / 2.0;
  p_value = gammq(a,y);

  return p_value;
}

double         calc_statistic_by_chi_square(int df, double p_value)
{
  double   x, a, y;
  double   min_chi = 0.0;
  double   max_chi = CHI_MAX;

  if      (p_value <= 0.0) {
    return max_chi;
  }
  else if (p_value >= 1.0) {
    return 0.0;
  }

  x = (double)df / sqrt(p_value);
  a = (double)df / 2.0;
  while (max_chi - min_chi > CHI_EPSILON) {
    y       = x  / 2.0;
    if (gammq(a,y) < p_value) {
      max_chi = x;
    }
    else {
      min_chi = x;
    }
    x = (max_chi + min_chi) * 0.5;
  }

  return x;
}

