#include "haplotypetest.h"

/* Error handler */
extern int            error_check();
extern int            error_message(char *);
extern int            warning_message(char *);

/* Define functions */
double         calc_PIC(int, double *);
double         calc_Heterozygosity(int, int, double *, double *);
double         calc_HWE_statistics(int *);

double         calc_PIC(int     number_of_allele_type,
			double *f_allele)
{
  double  PIC, pii, pij;
  int     i, j;

  pii = 0.0;
  pij = 0.0;

  /* sum pi^2 */
  for (i=0; i<number_of_allele_type; i++) {
    pii = pii + *(f_allele+i) * *(f_allele+i);
  }

  /* sum sum (pi*pj)^2 */
  for (i=0; i<number_of_allele_type-1; i++) {
    for (j=i+1; j<number_of_allele_type; j++) {
      pij = pij + 2.0 * *(f_allele+i) * *(f_allele+j) * *(f_allele+i) * *(f_allele+j);
    }
  }

  PIC = 1.0 - pii - pij;

  return PIC;
}

double         calc_Heterozygosity(int     number_of_allele_type,
				   int     number_of_allele,
				   double *f_allele,
				   double *var)
{
  double  het;
  double  pii, piii, N, biased;
  int     i;

  pii  = 0.0;
  piii = 0.0;

  /* sum pi^2 */
  for (i=0; i<number_of_allele_type; i++) {
    pii  = pii  + *(f_allele+i) * *(f_allele+i);
    piii = piii + *(f_allele+i) * *(f_allele+i) * *(f_allele+i);
  }

  N = (double)number_of_allele;

  /* Heterozygosity */ 
  biased = 1.0 - pii;
  het    = N * biased / (N - 1.0);

  /* Variance */
  biased = 2.0*(N-1.0) * ((3.0-2.0*N)*pii*pii + 2.0*(N-2.0)*piii + pii) / (N*N*N);
  *var   = N*N * biased / ((N-1.0)*(N-1.0)) ;

  return het;
}

double         calc_HWE_statistics(int *genotype_count)
{
  double  stat, g11, g12, g22, g;


  g11  = (double)*genotype_count;
  genotype_count++;
  g12  = (double)*genotype_count;
  genotype_count++;
  g22  = (double)*genotype_count;

  if ((g12 != 0) || (g22 != 0)) {
    g    = g11+g12+g22;
    g12  = g12/2.0;
    stat = g * (g12*g12 - g11*g22) * (g12*g12 - g11*g22)/((g11+g12) * (g22+g12) * (g11+g12) * (g22+g12));
  }
  else {
    stat = 0.0;
  }

  return stat;
}

