#define _MAIN_
#include "haplotypetest.h"

/* Option file processing */
extern int            init_option(Option_Data *);
extern int            get_option(char *, Option_Data *);

/* Genotype file processing */
extern Genotype_Data *get_genotype(char *, Option_Data *);
extern void           free_genotype(Genotype_Data *);

/* Phenotype file processing */
extern int            get_QTLvalue(Genotype_Data *, Option_Data *, char *);

/* Haplotype frequency inference */
extern Estimate_Set  *select_loci(Genotype_Data *, Option_Data *);
extern int            set_target_haplotype(Genotype_Data *, Estimate_Set *, Haplotype_Set *, Option_Data *);
extern int            estimate_haplotype(Genotype_Data *, Estimate_Set *, Option_Data *);

/* Association test */
extern Haplotype_Set *set_haplotype_set(Genotype_Data *, Estimate_Set *, Option_Data *);
extern Haplotype_Set *alloc_haplotype_set();
extern int            estimate_haplotype_and_penetrance(Genotype_Data *, Estimate_Set *, Haplotype_Set *, Option_Data *, Window_Result *);
extern int            estimate_haplotype_and_QTL(Genotype_Data *, Estimate_Set *, Haplotype_Set *, Option_Data *, Window_Result *);
extern void           save_non_zero_haplotype_set(Estimate_Set *, Option_Data *);
extern void           free_estimate_set(Estimate_Set *);
extern void           free_haplotype_set(Haplotype_Set *);

/* Sliding window test */
extern int            sliding_window_test(Genotype_Data *, Option_Data *);

/* Linkage disequilibrium measure */
extern int            estimate_LD_measure(Genotype_Data *, Option_Data *);

/* Utility function */
extern void          *mem_alloc(size_t, const char *);

/* Error handler */
extern int            reset_error();
extern int            error_check();
extern int            error_message(char *);
extern int            warning_message(char *);

/* Random generator */
extern void        set_random_seed(int);

/* Define functions */
int   interactive_mode(Option_Data *, char *, char *, char *);
int   batch_mode(Option_Data *, char *, char *, char *);

main(int argc, char *argv[])
{
  int             i;
  char            option_file[FILENAME_MAX] = "";
  char            genotype_file[FILENAME_MAX] = "";
  char            phenotype_file[FILENAME_MAX] = "";
  char            haplotype_freq_file[FILENAME_MAX] = "";
  int             message_opt = 0, freq_opt = 0;

  /* Option for calculation */
  Option_Data    *option;

  /* Option data initialize */
  option = (Option_Data *)mem_alloc(sizeof(Option_Data),"Option data structure");
  init_option(option);

  File_Status = (File_Status_data *)mem_alloc(sizeof(File_Status_data),"File status data structure");
  /* Argument analysis */
  for (i=1; i<argc; i++) {
    if      (strcmp("-f",argv[i]) == 0) {
      i++;
      if (i<argc) {
	strcpy(option_file, argv[i]);
      }
    }
    else if (strcmp("-g",argv[i]) == 0) {
      i++;
      if (i<argc) {
	strcpy(genotype_file, argv[i]);
      }
    }
    else if (strcmp("-p",argv[i]) == 0) {
      i++;
      if (i<argc) {
	strcpy(phenotype_file, argv[i]);
	option->penetrance = 1;
      }
    }
    else if (strcmp("-Q",argv[i]) == 0) {
      i++;
      if (i<argc) {
	strcpy(phenotype_file, argv[i]);
	option->QTL = 1;
      }
    }
    else if (strcmp("-d",argv[i]) == 0) {
      i++;
      if (i<argc) {
	strcpy(haplotype_freq_file, argv[i]);
	freq_opt = 1;
      }
    }
    else if (strcmp("-v",argv[i]) == 0) {
      message_opt = 1;
    }
  }

  /* Option data processing */
  option->message    = message_opt;
  option->freq_table = freq_opt;

  get_option(option_file, option);
  error_check();
  /* for debug */
  /*File_Status->interactive = 1;*/

  if (File_Status->interactive == 1) {
    interactive_mode(option, genotype_file, phenotype_file, haplotype_freq_file);
  }
  else {
    batch_mode(option, genotype_file, phenotype_file, haplotype_freq_file);
  }

  free(option);
  free(File_Status);

  /* Finish */
  exit(0);
}

int   interactive_mode(Option_Data *option,
		       char        *genotype_file,
		       char        *phenotype_file,
		       char        *haplotype_freq_file)
{
  return 0;
}

int   batch_mode(Option_Data *option,
		 char        *genotype_file,
		 char        *phenotype_file,
		 char        *haplotype_freq_file)
{
  Genotype_Data  *genotype;
  Estimate_Set   *table_for_EM, *table_for_PEN, *table_for_QTL;
  Haplotype_Set  *set_of_haplotype;

  /* Genotype data processing */
  genotype = get_genotype(genotype_file, option);

  if (option->freq_table) {
    
  }

  set_random_seed(RANDOM_SEED);

  /* Linkage disequilibrium measure */
  if (option->pairLD) {
    printf("\n=== Linkage disequilibrium measure");
    estimate_LD_measure(genotype,option);
  }

  /* Sliding window test */
  if (option->window_flag) {
    printf("\n=== Sliding window test ===\n");
    if (option->penetrance) {
      get_phenotype(genotype, option, phenotype_file);
    }
    if (option->QTL) {
      get_QTLvalue(genotype, option, phenotype_file);
    }

    sliding_window_test(genotype, option);
  }

  /* Normal calculation */
  else {
  /* Haplotype inference */
    if (option->haplotype || option->htSNP) {
      /* Message */
      printf("\n=== Haplotype inference ===\n");

      /* Select loci */
      table_for_EM = select_loci(genotype, option);

      /* Haplotype inference by EM */
      estimate_haplotype(genotype,table_for_EM,option);

      if (option->htSNP) {
	/* Message */
	printf("\n=== htSNP and SNP anchored haplotype set calculation ===\n");

	/* Save nonzero haplotypes */
	save_non_zero_haplotype_set(table_for_EM,option);

	set_of_haplotype = set_haplotype_set(genotype,table_for_EM,option);
      }
      else {
	/* Free haplotype inference table */
	free_estimate_set(table_for_EM);
      }
    }

    if (option->penetrance) {
      /* Message */
      printf("\n=== Simultaneous inference of haplotypes and penetrances ===\n");
      get_phenotype(genotype, option, phenotype_file);

      /* Select loci */
      table_for_PEN = select_loci(genotype, option);

      /* Target haplotype decode */
      if (!option->haplotype_set) {
	set_of_haplotype = alloc_haplotype_set();
	set_target_haplotype(genotype,table_for_PEN,set_of_haplotype,option);
      }

      /* Haplotype and penetrance inference by EM */
      estimate_haplotype_and_penetrance(genotype,table_for_PEN,set_of_haplotype,option,NULL);

      /* Free haplotype inference table */
      free_estimate_set(table_for_PEN);
    }

    if (option->QTL) {
      /* Message */
      printf("\n=== Simultaneous inference of haplotypes and QTL parameters ===\n");

      /* Read QTL value */
      get_QTLvalue(genotype, option, phenotype_file);

      /* Select loci */
      table_for_QTL = select_loci(genotype, option);

      /* Target haplotype decode */
      if (!option->haplotype_set) {
	set_of_haplotype = alloc_haplotype_set();
	set_target_haplotype(genotype,table_for_QTL,set_of_haplotype,option);
      }

      /* Haplotype and QTL parameter inference by EM */
      estimate_haplotype_and_QTL(genotype,table_for_QTL,set_of_haplotype,option,NULL);

      /* Free haplotype inference table */
      free_estimate_set(table_for_QTL);
    }

    if (option->htSNP) {
      free_estimate_set(table_for_EM);
      free_haplotype_set(set_of_haplotype);
    }
  }

  /* Free data area */
  free_genotype(genotype);

  printf("\nExecution successfully ended\n");

  return 0;
}
