#include "haplotypetest.h"

/* Utility function */
extern void          *mem_alloc(size_t, const char *);

/* Error handler */
extern int            error_check();
extern int            error_message(char *);
extern int            warning_message(char *);

/* Likelihood function */
extern double         Total_likelihood(Estimate_Set *, double *);
extern double         Total_likelihood_with_penetrance(Estimate_Set *, double *);
extern double         Total_likelihood_with_QTL(Estimate_Set *, Genotype_Data *, double *);

/* Define functions */
double         noLD_likelihood(Estimate_Set *, double *, int);
double         noLD_likelihood_with_penetrance(Estimate_Set *, double *, int);
double         noLD_likelihood_with_QTL(Estimate_Set *, Genotype_Data *, double *, int);
int            noLD_frequency(Estimate_Set *, double *, double *);

double         noLD_likelihood(Estimate_Set *table,
			       double       *f,
			       int           message)
{
  /* likelihood */
  double       likelihood;

  /* Likelihood assuming no linkage disequilibrium */
  likelihood = Total_likelihood(table, f);

  if (message) {
    printf("log10 likelihood assuming no linkage disequilibrium = %13.6e\n",likelihood);
  }

  return likelihood;
}

double         noLD_likelihood_with_penetrance(Estimate_Set *table,
					       double       *f,
					       int           message)
{
  /* likelihood */
  double       likelihood;

  /* Likelihood assuming no linkage disequilibrium */
  likelihood = Total_likelihood_with_penetrance(table, f);

  if (message) {
    printf("log10 likelihood assuming no linkage disequilibrium = %13.6e\n",likelihood);
  }

  return likelihood;
}

double         noLD_likelihood_with_QTL(Estimate_Set *table,
					Genotype_Data *genotype,
					double       *f,
					int           message)
{
  /* likelihood */
  double       likelihood;

  /* Likelihood assuming no linkage disequilibrium */
  likelihood = Total_likelihood_with_QTL(table, genotype, f);

  if (message) {
    printf("log10 likelihood assuming no linkage disequilibrium = %13.6e\n",likelihood);
  }

  return likelihood;
}

int            noLD_frequency(Estimate_Set *table,
			      double       *f_allele,
			      double       *f_noLD)
{
  /* Local variables */
  haplotype  h, haplo;
  int        l, a;
  double     f, allele_p;

  for (h=0; h<table->number_of_haplo; h++) {
    f     = 1.0;
    haplo = *(table->haplo + h);
    for (l=0; l<table->select_loci; l++) {
      a        = (haplo >> (table->select_loci-l-1)) & 01;
      if (a) {
	allele_p = 1.0 - *(f_allele + l);
      }
      else {
	allele_p = *(f_allele + l);
      }
      f = f * allele_p;
    }
    *(f_noLD+h) = f;
  }

  return 0;
}
